///**
// * Copyright (C) 2012 - 101loops.com <dev@101loops.com>
// *
// * Licensed under the Apache License, Version 2.0 (the "License");
// * you may not use this file except in compliance with the License.
// * You may obtain a copy of the License at
// *
// *         http://www.apache.org/licenses/LICENSE-2.0
// *
// * Unless required by applicable law or agreed to in writing, software
// * distributed under the License is distributed on an "AS IS" BASIS,
// * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// * See the License for the specific language governing permissions and
// * limitations under the License.
// */
//package com.crashnote.core.report.impl.session;
//
//import com.crashnote.core.model.data.BoundedList;
//import com.crashnote.core.model.log.LogEvt;
//
//import java.util.*;
//
///**
// * This class represents a collection of log events, whereby exceptions are managed separately
// * from regular events. The size of the collection is finite and depends on the configuration.
// * <p/>
// * By using this class, the library can remember the last few events before a crash in order to
// * provide additional insight about its cause.
// */
//public class LogHeap {
//
//    /**
//     * list of exceptions
//     */
//    private final List<LogEvt<?>> excps;
//
//    /**
//     * list of non-exceptions
//     */
//    private final List<LogEvt<?>> history;
//
//    // SETUP ======================================================================================
//
//    public LogHeap(final int historySize) {
//        this.excps = new ArrayList<LogEvt<?>>(5);
//
//        if (historySize < 1)
//            this.history = null;
//        else // use bounded list to limit element count
//            this.history = new BoundedList<LogEvt<?>>(historySize);
//    }
//
//    /**
//     * Copy-Constructor
//     */
//    public LogHeap(final LogHeap heap) {
//        // make a copy of the excp list
//        this.excps = new ArrayList<LogEvt<?>>(heap.excps);
//
//        // make a copy of the history
//        if (heap.history != null) this.history = new ArrayList<LogEvt<?>>(heap.history);
//        else this.history = null;
//
//        // and then defer this
//        defer();
//    }
//
//    // INTERFACE ==================================================================================
//
//    /**
//     * By deferring the local members, copies of the references they contain are created
//     * - thus they can be processed in a separate thread
//     */
//    public void defer() {
//        for (final LogEvt e : excps) e.defer();
//    }
//
//    /**
//     * Once the heap was sent, move all the exceptions to the history
//     */
//    public void sent() {
//        for (final LogEvt e : excps) {
//            excps.remove(e);
//            history.add(e);
//        }
//    }
//
//    public boolean isEmpty() {
//        return excps.isEmpty() && (history == null || history.isEmpty());
//    }
//
//    public void addEvt(final LogEvt<?> evt) {
//        if (evt.isExcp())
//            excps.add(evt);
//        else if (history != null)
//            history.add(evt);
//    }
//
//    // GET ========================================================================================
//
//    public LogEvt<?>[] getEvents() {
//        int i = 0;
//        final LogEvt<?>[] res = new LogEvt<?>[getExcpCount() + getHistorySize()];
//        if (history != null) for (final LogEvt<?> e : history) res[i++] = e;
//        for (final LogEvt<?> e : excps) res[i++] = e;
//        return res;
//    }
//
//    public int getExcpCount() {
//        return excps.size();
//    }
//
//    public int getHistorySize() {
//        return (history == null) ? 0 : history.size();
//    }
//}
